import re

from werkzeug.urls import url_parse

from odoo import _
from odoo.exceptions import RedirectWarning
from odoo.addons.saas_worker import metabase
from odoo.addons.saas_worker.const import WHATSAPP_TRIAL_PHONE_UID, WHATSAPP_TRIAL_TOKEN
from odoo.addons.whatsapp.tools.whatsapp_api import WhatsAppApi
from odoo.addons.whatsapp.tools.whatsapp_exception import WhatsAppError
from odoo.tools import TEXT_URL_REGEX as URL_REGEX

import logging
_logger = logging.getLogger(__name__)


INC_BY_WHATSAPP = 1
MAX_TEXT_LENGTH = 30

PREFIX_URLS = [
    '/my/invoice',
    '/my/order',
    '/my/purchase',
    '/contactus',
]

def monkey_patch(cls):
    """ Return a method decorator to monkey-patch the given class. """
    def decorate(func):
        name = func.__name__
        func.super = getattr(cls, name, None)
        setattr(cls, name, func)
        return func
    return decorate


@monkey_patch(WhatsAppApi)
def __init__(self, wa_account_id):
    __init__.super(self, wa_account_id)
    if wa_account_id.phone_uid == 'odoo_demo_account':
        self.phone_uid = WHATSAPP_TRIAL_PHONE_UID
        self.token = WHATSAPP_TRIAL_TOKEN
        self.is_shared_account = True

@monkey_patch(WhatsAppApi)
def _test_connection(self):
    if self.wa_account_id.phone_uid == 'odoo_demo_account':
        return
    return _test_connection.super(self)

@monkey_patch(WhatsAppApi)
def _send_whatsapp(self, number, message_type, send_vals, parent_message_id=False):
    if self.wa_account_id.phone_uid != 'odoo_demo_account':
        return _send_whatsapp.super(self, number, message_type, send_vals, parent_message_id)

    if message_type != 'template':
        self.env['whatsapp.template'].raise_demo_warning('[NO_TEMPLATE]')

    db = metabase.get(self.wa_account_id.env.cr.dbname)
    inc = INC_BY_WHATSAPP
    if db.email_daily_count + inc > db.email_daily_limit:
        raise RedirectWarning(_("""
To protect our customers, we have imposed a limit on outgoing messages for the WhatsApp Demo Account.
You have reached your daily messages limit.

To continue sending more WhatsApp messages, we recommend creating your own WhatsApp Business
Account by following our documentation.

Benefits of using your own WhatsApp account include:
    - Displaying your business name in WhatsApp with a green tick
    - Receiving acknowledgment of message delivery and read status
    - Creating your own message templates
    - Receiving replies from customers
    - Responding to customer messages
    - Getting reactions to your messages
    - Sending and receiving images, videos, documents, and more.
    - ...
        """), {
            'type': 'ir.actions.act_url',
            'url': 'https://www.odoo.com/r/whatsapp-doc',
            'target': 'new'
        }, _("View documentation"))

    for component in send_vals.get('components'):
        for parameter in component.get('parameters'):
            if parameter.get('text'):
                text = parameter['text']
                urls = re.findall(URL_REGEX, text)
                if urls:
                    u = url_parse(text)
                    # naive check... mydb.odoo.com/r or redirect in content works too ;)
                    for host in metabase.get_hosts(db):
                        if host == u.netloc:  # domain match
                            if any([u.path.startswith(preurl) for preurl in PREFIX_URLS]):  # path match
                                break
                    else:
                        _logger.error("WhatsApp url blocked: %s", text)
                        raise WhatsAppError(_("""
                            The presence of an unauthorized link has been detected in the message.
                            This is a limitation of the WhatsApp Demo Account."""), failure_type='account')
                elif len(text) > MAX_TEXT_LENGTH:
                    raise WhatsAppError(_("""
                        The message contains a variable value that exceeds the allowed length.
                        This is a limitation of the WhatsApp Demo Account."""), failure_type='account')

    metabase.increment(db, email_daily_count=INC_BY_WHATSAPP)
    return _send_whatsapp.super(self, number, message_type, send_vals, parent_message_id)
